import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.io.PrintWriter;
import java.io.FileWriter;

/* These two filters select for files containing one puzzle (*.sud) and multiple puzzles (*.sdb) */
class SudokuFileFilter extends FileFilter {

	public String getDescription() {
		return "File of Sudoku problems";
	}
	
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		}
		
		if (f.getName().length() < 3) {
			return false;
		}
		
		String extension = f.getName().substring(f.getName().length() - 4, f.getName().length());
		if (extension != null) {
			if (extension.equals(".sud")) {
				return true;
			} else {
				return false;
			}
		}
	
		return false;
	}
}

class SudokuDatabaseFilter extends FileFilter {

	public String getDescription() {
		return "Database of Sudoku problems";
	}
	
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		}
		
		if (f.getName().length() < 3) {
			return false;
		}
		
		String extension = f.getName().substring(f.getName().length() - 4, f.getName().length());
		if (extension != null) {
			if (extension.equals(".sdb")) {
				return true;
			} else {
				return false;
			}
		}
	
		return false;
	}
}

/* As some menu items are dynamically generated I need a way to tell which one called the event */
class TagMenuItem extends JMenuItem {
	int Tag;
	
	public TagMenuItem(String S) {
		super(S);
	}
	
	public void setTag(int T) {
		Tag = T;
	}
	
	public int getTag() {
		return Tag;
	}
}

/* This does all the display stuff */
public class Display implements ActionListener {
	JMenuBar MenuBar;
	MainPanel mainPanel;
	JFrame frame;
	JPanel StatusBar;
	JLabel ModeLabel;
	IndexPanel ImagesIndex;
	static Vector ImageFiles;
	JMenu ImagesSubMenu;
	JMenuItem ThreeByThreeMI, FourByFourMI, FiveByFiveMI, TwoByTwoMI, FourByTwoMI;
	JMenuItem HelpMI, AboutMI;
	JMenuItem OptionsMI;
	JMenuItem ViewMI, LoadMI, AddMI, ImportMI, SaveDBMI;
	int PathEnd;
	
	int Type = 0;
	int ImagesLength = 11;
	
	String ImageLocation;
	
	/* ImageLocation is the base of the image directories */
	public String getImageLocation() {
		return ImageLocation;
	}
	
	/* The first images are used as the initial set of images */
	public static String[] getFirstImages() {
		return (String[]) ImageFiles.firstElement();
	}
	
	public void setImageLocation(String s) {
		ImageLocation = s;
	}
	
	/* This saves the config file. I felt it was easier to regenerate the whole thing rather than try to modify it */
	public void saveConfig() {
		String[] CurrentImages;
		File ConfigFile = new File("asudoku.conf");
		try {
			PrintWriter pw = new PrintWriter(new FileWriter(ConfigFile, false));
			pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			pw.println("<Wrapper>");
			pw.println("<ImageLocation>");
			pw.println(ImageLocation);
			pw.println("</ImageLocation>");
			for (Enumeration e = ImageFiles.elements(); e.hasMoreElements() ;) {
				CurrentImages = (String[]) e.nextElement();
				pw.println("<Images>");
				pw.println("<ImageTitle>");
				pw.println(CurrentImages[0]);
				pw.println("</ImageTitle>");
				/* Separating the path and filename makes it easier to move images */
				pw.println("<ImagePath>");
				PathEnd = CurrentImages[1].lastIndexOf(File.separator);
				pw.println(CurrentImages[1].substring(0, PathEnd + 1));
				pw.println("</ImagePath>");
				for (int i = 1; i < CurrentImages.length; i++) {
					pw.println("<Image>");
					pw.println(CurrentImages[i].substring(PathEnd + 1, CurrentImages[i].length()));
					pw.println("</Image>");
				}
				pw.println("</Images>");
			}		
			pw.println("</Wrapper>");
			pw.flush();
			pw.close();
		} catch (Exception e) {
			System.out.println("Falied to write config file");
		}
	}
	
	/* Used to initialise the Images after being read in from the config file */
	public static void setAllImagePaths(Vector Paths) {
		ImageFiles = Paths;
	}
	
	/* Mostly about adding a new menu item */
	public void addImageFiles(String[] Images) {
		TagMenuItem NewImageOption = new TagMenuItem(Images[0]);
		NewImageOption.setTag(ImageFiles.size());
		NewImageOption.addActionListener(ImageListener);
		ImagesSubMenu.add(NewImageOption);
		ImageFiles.add(Images);
	}
	
	/* Mostly devoted to setting up all the menu item */
	public Display() {
		mainPanel = new MainPanel(this);
		JMenu FileMenu, ModeMenu, OptionMenu, TypeSubMenu, HelpMenu, DatabaseMenu;;
		JMenuItem QuitMI, OpenMI, SaveMI, PrintMI;
		JMenuItem InputMI, SolveMI, GenerateMI;
		JMenuItem ColourMI, DigitsMI;
		
		char[] fileShortcuts = {'O', 'P', 'S', 'Q'};
		char[] modeShortcuts = {'V', 'I', 'G'};
		char[] helpShortcuts = {'H', 'A'};
		
		MenuBar = new JMenuBar();
		FileMenu = new JMenu("File");
		MenuBar.add(FileMenu);
		
		OpenMI = new JMenuItem("Open", fileShortcuts[0]);
		OpenMI.addActionListener(openListener);
		FileMenu.add(OpenMI);
		
		PrintMI = new JMenuItem("Print", fileShortcuts[1]);
		PrintMI.addActionListener(printListener);
		FileMenu.add(PrintMI);
		
		SaveMI = new JMenuItem("Save", fileShortcuts[2]);
		SaveMI.addActionListener(saveListener);
		FileMenu.add(SaveMI);
		
		QuitMI = new JMenuItem("Quit", fileShortcuts[3]);
		QuitMI.addActionListener(quitListener);
		FileMenu.add(QuitMI);
		
		ModeMenu = new JMenu("Mode");
		MenuBar.add(ModeMenu);
		
		SolveMI = new JMenuItem("Solve", modeShortcuts[0]);
		SolveMI.addActionListener(solveListener);
		ModeMenu.add(SolveMI);

		InputMI = new JMenuItem("Input", modeShortcuts[1]);
		InputMI.addActionListener(inputListener);
		ModeMenu.add(InputMI);

		GenerateMI = new JMenuItem("Generate", modeShortcuts[2]);
		GenerateMI.addActionListener(generateListener);
		ModeMenu.add(GenerateMI);
		
		OptionMenu = new JMenu("Options");
		MenuBar.add(OptionMenu);
		
		ImagesSubMenu = new JMenu("Images");
		OptionMenu.add(ImagesSubMenu);
		
		int TagIndex = 0;
		for (Enumeration e = ImageFiles.elements(); e.hasMoreElements() ;) {
			TagMenuItem NewImageOption = new TagMenuItem(((String[]) e.nextElement())[0]);
			NewImageOption.setTag(TagIndex++);
			NewImageOption.addActionListener(ImageListener);
			ImagesSubMenu.add(NewImageOption);
		}
		
		TypeSubMenu = new JMenu("Puzzle Type");
		
		TwoByTwoMI = new JMenuItem("2x2");
		TwoByTwoMI.addActionListener(typeListener);
		TypeSubMenu.add(TwoByTwoMI);
		
		ThreeByThreeMI = new JMenuItem("3x3");
		ThreeByThreeMI.addActionListener(typeListener);
		TypeSubMenu.add(ThreeByThreeMI);

		FourByFourMI = new JMenuItem("4x4");
		FourByFourMI.addActionListener(typeListener);
		TypeSubMenu.add(FourByFourMI);

		FiveByFiveMI = new JMenuItem("5x5");
		FiveByFiveMI.addActionListener(typeListener);
		TypeSubMenu.add(FiveByFiveMI);
		
		FourByTwoMI = new JMenuItem("4x2");
		FourByTwoMI.addActionListener(typeListener);
		TypeSubMenu.add(FourByTwoMI);
		
		OptionMenu.add(TypeSubMenu);
		
		OptionsMI = new JMenuItem("More Options");
		OptionsMI.addActionListener(this);
		
		OptionMenu.add(OptionsMI);
		
		DatabaseMenu = new JMenu("Database");
		MenuBar.add(DatabaseMenu);
	
		/* It shouldn't be possible to do anything with the database until one has been loaded */
		/* Maybe need a way to create a new database? Should be created by loading an empty one? */
		AddMI = new JMenuItem("Add Current Puzzle");
		AddMI.addActionListener(databaseListener);
		AddMI.setEnabled(false);
		DatabaseMenu.add(AddMI);
			
		ImportMI = new JMenuItem("Import Database");
		ImportMI.addActionListener(databaseListener);
		ImportMI.setEnabled(false);
		DatabaseMenu.add(ImportMI);
			
		LoadMI = new JMenuItem("Load Database");
		LoadMI.addActionListener(databaseListener);
		DatabaseMenu.add(LoadMI);
	
		SaveDBMI = new JMenuItem("Save Database");
		SaveDBMI.addActionListener(databaseListener);
		SaveDBMI.setEnabled(false);
		DatabaseMenu.add(SaveDBMI);
	
		ViewMI = new JMenuItem("View Database");
		ViewMI.addActionListener(databaseListener);
		ViewMI.setEnabled(false);
		DatabaseMenu.add(ViewMI);
	
		HelpMenu = new JMenu("Help");
		MenuBar.add(HelpMenu);
		
		HelpMI = new JMenuItem("ASudoku Help", helpShortcuts[0]);
		HelpMI.addActionListener(helpListener);
		HelpMenu.add(HelpMI);
		
		AboutMI = new JMenuItem("About", helpShortcuts[1]);
		AboutMI.addActionListener(helpListener);
		HelpMenu.add(AboutMI);
		
		StatusBar = new JPanel();
		StatusBar.setLayout(new BoxLayout(StatusBar, BoxLayout.X_AXIS));
		ModeLabel = new JLabel("Mode: Input");
		ImagesIndex = new IndexPanel();
		StatusBar.add(ModeLabel);
		StatusBar.add(ImagesIndex);
		
		frame = new JFrame("ASudoku");
		frame.setJMenuBar(MenuBar);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.getContentPane().add(mainPanel, BorderLayout.CENTER);
		frame.getContentPane().add(StatusBar, BorderLayout.SOUTH);
		frame.pack();	
	
		frame.setVisible(true);	
		
		/* Means the database will be closed when exiting the program */
		frame.addWindowListener(new WindowAdapter() {
        	public void windowClosing(WindowEvent we) {
            	addWindowListener_windowClosing(we);
        	}
    	});
	}
	
	private void addWindowListener_windowClosing(WindowEvent e){
        Database.saveDatabase();
        System.exit(0);
    }
	
	public void setModeText(String S) {
		ModeLabel.setText(S);
	}
	
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == OptionsMI) {
			OptionScreen OS = new OptionScreen(this);
		}
	}
	
	ActionListener helpListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == AboutMI) {
				HelpAboutScreen HAS = new HelpAboutScreen();
			}
			if (event.getSource() == HelpMI) {
				HelpScreen HS = new HelpScreen();
			}
		}
	};
	
	ActionListener databaseListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == SaveDBMI) {
				Database.saveDatabase();
			}
			if (event.getSource() == ImportMI) {
				ReadFile rf = new ReadFile();
				JFileChooser chooser = new JFileChooser();
				SudokuDatabaseFilter filter = new SudokuDatabaseFilter();
				chooser.setFileFilter(filter);
				Puzzle CurrentPuzzle;
				int option = chooser.showOpenDialog(MenuBar);
		
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					try {
						Vector Puzzles = rf.Read(sf);
						for (Enumeration e = Puzzles.elements(); e.hasMoreElements() ;) {
							CurrentPuzzle = (Puzzle) e.nextElement();
							try { 
								if (!(CurrentPuzzle.solvePuzzle())) {
									Puzzles.remove(CurrentPuzzle);
								}
							}
							catch (InsolublePuzzleError ipe) {
								Puzzles.remove(CurrentPuzzle);
							}
						}
						Database.add(Puzzles);
						JOptionPane.showMessageDialog(frame, 
									"The database was successfully imported",
									"Imported Database",
									JOptionPane.INFORMATION_MESSAGE);
					}
					catch (InvalidXMLError ixe) {
						JOptionPane.showMessageDialog(frame,
							"This database is corrupted, or is not a proper database",
							"Invalid Database",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
			
			if (event.getSource() == AddMI) {
				Database.addPuzzle(mainPanel.getPuzzle());
			}
			if (event.getSource() == ViewMI) {
				DatabaseScreen DS = new DatabaseScreen(mainPanel);
			}
			if (event.getSource() == LoadMI) {
				ReadFile rf = new ReadFile();
				JFileChooser chooser = new JFileChooser();
				SudokuDatabaseFilter filter = new SudokuDatabaseFilter();
				chooser.setFileFilter(filter);
				Puzzle CurrentPuzzle;
				int option = chooser.showOpenDialog(MenuBar);
		
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					try {
						Vector Puzzles = rf.Read(sf);
						for (Enumeration e = Puzzles.elements(); e.hasMoreElements() ;) {
							CurrentPuzzle = (Puzzle) e.nextElement();
							try { 
								if (!(CurrentPuzzle.solvePuzzle())) {
									Puzzles.remove(CurrentPuzzle);
								}
							}
							catch (InsolublePuzzleError ipe) {
								Puzzles.remove(CurrentPuzzle);
							}
						}
						Database.add(Puzzles);
						/* Sets the filename of the database that is active */
						Database.setFile(sf);
						/* Now we have a database to play with we allow the user to do things with it */
						ImportMI.setEnabled(true);
						ViewMI.setEnabled(true);
						AddMI.setEnabled(true);
						SaveDBMI.setEnabled(true);
						JOptionPane.showMessageDialog(frame, 
									"The database was successfully loaded",
									"Loaded Database",
									JOptionPane.INFORMATION_MESSAGE);
					}
					catch (InvalidXMLError ixe) {
						JOptionPane.showMessageDialog(frame,
							"This database is corrupted, or is not a proper database",
							"Invalid Database",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}
	};
	
	ActionListener openListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			ReadFile rf = new ReadFile();
			JFileChooser chooser = new JFileChooser();
			SudokuFileFilter filter = new SudokuFileFilter();
    		chooser.setFileFilter(filter);
    		Puzzle CurrentPuzzle;
   			int option = chooser.showOpenDialog(MenuBar);
	
			if (option == JFileChooser.APPROVE_OPTION) {
				File sf = chooser.getSelectedFile();
				try {
					Vector Puzzles = rf.Read(sf);
					for (Enumeration e = Puzzles.elements(); e.hasMoreElements() ;) {
						CurrentPuzzle = (Puzzle) e.nextElement();
						try { 
							if (!(CurrentPuzzle.solvePuzzle())) {
								Puzzles.remove(CurrentPuzzle);
							}
						}
						catch (InsolublePuzzleError ipe) {
							Puzzles.remove(CurrentPuzzle);
						}
					}
					/* If more than one puzzle ask which one to use */
					if (Puzzles.size() == 1) {
						if (((Puzzle) Puzzles.firstElement()).AttemptedExists()) {
							/* Start fresh or from last time */
							int check = JOptionPane.showConfirmDialog(null,
										"Do you want to continue from last time?",
								 		"Use Attempted?", JOptionPane.YES_NO_OPTION);
							if (check == JOptionPane.YES_OPTION) {
								mainPanel.addPuzzle((Puzzle) Puzzles.firstElement(), true);
							} else {
								mainPanel.addPuzzle((Puzzle) Puzzles.firstElement(), false);
							}
						} else {
							mainPanel.addPuzzle((Puzzle) Puzzles.firstElement(), false);
						}
					} else {
					
						if (Puzzles.size() > 1) {
							Object[] possibilities = new Object[Puzzles.size()];
							for (int i = 0; i < Puzzles.size(); i++) {
								possibilities[i] = Integer.toString(i);
							}
							String s = (String)JOptionPane.showInputDialog(
                    			frame,
								"There are " + Puzzles.size() + " puzzles. Which one do you want?",
								"Select Puzzle",
			                    JOptionPane.PLAIN_MESSAGE,
            			        null,
            			        possibilities,
            		       		"1");
							if ((s != null) && (s.length() > 0)) {
								if (((Puzzle) Puzzles.firstElement()).AttemptedExists()) {
									int check = JOptionPane.showConfirmDialog(null,
												"Do you want to continue from last time?",
										 		"Use Attempted?", JOptionPane.YES_NO_OPTION);
									if (check == JOptionPane.YES_OPTION) {
										mainPanel.addPuzzle((Puzzle) Puzzles.get(Integer.parseInt(s)), true);
									} else {
										mainPanel.addPuzzle((Puzzle) Puzzles.get(Integer.parseInt(s)), false);
									}
								} else {
									mainPanel.addPuzzle((Puzzle) Puzzles.get(Integer.parseInt(s)), false);
								}
							}

						} else {
							JOptionPane.showMessageDialog(frame,
    							"This file does not contain a valid Sudoku puzzle",
    							"Invalid File",
    							JOptionPane.ERROR_MESSAGE);
    					}
					}
				}
				catch (InvalidXMLError ixe) {
					JOptionPane.showMessageDialog(frame,
    					"This file does not contain a valid Sudoku puzzle",
    					"Invalid File",
    					JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	};
	
	ActionListener saveListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			mainPanel.getPuzzle().setAttempted(mainPanel.getAttempted());
			SaveOptions SO = new SaveOptions(mainPanel.getPuzzle());
		}
	};
	
	ActionListener quitListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			Database.saveDatabase();
			System.exit(0);
		}
	};
	
	ActionListener solveListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			mainPanel.setState("solve");
		}
	};

	ActionListener inputListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			mainPanel.setState("input");
			frame.validate();
			frame.repaint();
		}
	};

	ActionListener generateListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			mainPanel.setState("generate");
			frame.validate();
			frame.repaint();
		}
	};
	
	ActionListener printListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			mainPanel.getPuzzle().setAttempted(mainPanel.getAttempted());
			PrintOptions PO = new PrintOptions(mainPanel.getPuzzle());
		}
	};
	
	ActionListener ImageListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			updateImages(((TagMenuItem) event.getSource()).getTag());
		}
	};

	private void updateImages(int Which) {
		String[] Images = (String[]) ImageFiles.get(Which);
		/* Images must be long enough. NumberOfDigits does not include blank, or the title of the images */
		if (Images.length > (PuzzleArrays.NumberOfDigits[Type] + 1)) {
			DigitPanel.changeImages(Images);
			mainPanel.updateImages();
			ImagesIndex.updateImages(Images.length - 2);
			ImagesLength = Images.length;
			frame.validate();
			frame.repaint();
		} else {
			JOptionPane.showMessageDialog(frame,
				"Sorry, but those images cannot be used for this size of puzzle",
				"Insufficient Images",
				JOptionPane.ERROR_MESSAGE);
		}
	}
	
	ActionListener typeListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			int Count = 0;
			String[] Element;
      	   	/* Switches to a different size. If the images are not suitable ask for a new set */
      	   	 if (event.getSource() == FourByTwoMI) {
				if (ImagesLength > (PuzzleArrays.NumberOfDigits[4] + 1)) {
					mainPanel.setType(Puzzle.FourXTwo);
					Type = 4;
					ImagesIndex.updateImages(PuzzleArrays.NumberOfDigits[Type]);
				} else {
					for (Enumeration e = ImageFiles.elements(); e.hasMoreElements();) {
						if (((String[]) e.nextElement()).length > (PuzzleArrays.NumberOfDigits[4] + 1)) {
							Count++;
						}
					}
					Object[] possibilities = new Object[Count];
					int i = 0;
					for (Enumeration e = ImageFiles.elements() ; e.hasMoreElements() ;) {
						Element = (String[]) e.nextElement();
						if (Element.length > (PuzzleArrays.NumberOfDigits[4] + 1)) {
							possibilities[i] = Element[0];
							i++;
						}
      	   			}
					getNewImages(Puzzle.FourXTwo, 4, possibilities);
				}
			}
      	   	
      	   	if (event.getSource() == TwoByTwoMI) {
				if (ImagesLength > (PuzzleArrays.NumberOfDigits[3] + 1)) {
					mainPanel.setType(Puzzle.TwoXTwo);
					Type = 3;
					ImagesIndex.updateImages(PuzzleArrays.NumberOfDigits[Type]);
				} else {
					for (Enumeration e = ImageFiles.elements(); e.hasMoreElements();) {
						if (((String[]) e.nextElement()).length > (PuzzleArrays.NumberOfDigits[3] + 1)) {
							Count++;
						}
					}
					Object[] possibilities = new Object[Count];
					int i = 0;
					for (Enumeration e = ImageFiles.elements() ; e.hasMoreElements() ;) {
						Element = (String[]) e.nextElement();
						if (Element.length > (PuzzleArrays.NumberOfDigits[3] + 1)) {
							possibilities[i] = Element[0];
							i++;
						}
      	   			}
					getNewImages(Puzzle.TwoXTwo, 3, possibilities);
				}
			}
      	   	
			if (event.getSource() == ThreeByThreeMI) {
				if (ImagesLength > (PuzzleArrays.NumberOfDigits[0] + 1)) {
					mainPanel.setType(Puzzle.ThreeXThree);
					Type = 0;
					ImagesIndex.updateImages(PuzzleArrays.NumberOfDigits[Type]);
				} else {
					for (Enumeration e = ImageFiles.elements(); e.hasMoreElements();) {
						if (((String[]) e.nextElement()).length > (PuzzleArrays.NumberOfDigits[0] + 1)) {
							Count++;
						}
					}
					Object[] possibilities = new Object[Count];
					int i = 0;
					for (Enumeration e = ImageFiles.elements() ; e.hasMoreElements() ;) {
						Element = (String[]) e.nextElement();
						if (Element.length > (PuzzleArrays.NumberOfDigits[0] + 1)) {
							possibilities[i] = Element[0];
							i++;
						}
      	   			}
					getNewImages(Puzzle.ThreeXThree, 0, possibilities);
				}
			}
			if (event.getSource() == FourByFourMI) {
				if (ImagesLength > (PuzzleArrays.NumberOfDigits[1] + 1)) {
					mainPanel.setType(Puzzle.FourXFour);
					Type = 1;
					ImagesIndex.updateImages(PuzzleArrays.NumberOfDigits[Type]);
				} else {
					for (Enumeration e = ImageFiles.elements(); e.hasMoreElements();) {
						if (((String[]) e.nextElement()).length > (PuzzleArrays.NumberOfDigits[1] + 1)) {
							Count++;
						}
					}
					Object[] possibilities = new Object[Count];
					int i = 0;
					for (Enumeration e = ImageFiles.elements() ; e.hasMoreElements() ;) {
						Element = (String[]) e.nextElement();
						if (Element.length > (PuzzleArrays.NumberOfDigits[1] + 1)) {
							possibilities[i] = Element[0];
							i++;
						}
      	   			}
					getNewImages(Puzzle.FourXFour, 1, possibilities);
				}
			}
			if (event.getSource() == FiveByFiveMI) {
				if (ImagesLength > (PuzzleArrays.NumberOfDigits[2] + 1)) {
					mainPanel.setType(Puzzle.FiveXFive);
					Type = 2;
					ImagesIndex.updateImages(PuzzleArrays.NumberOfDigits[Type]);
				} else {
					for (Enumeration e = ImageFiles.elements(); e.hasMoreElements();) {
						if (((String[]) e.nextElement()).length > (PuzzleArrays.NumberOfDigits[2] + 1)) {
							Count++;
						}
					}
					Object[] possibilities = new Object[Count];
					int i = 0;
					for (Enumeration e = ImageFiles.elements() ; e.hasMoreElements() ;) {
						Element = (String[]) e.nextElement();
						if (Element.length > (PuzzleArrays.NumberOfDigits[2] + 1)) {
							possibilities[i] = Element[0];
							i++;
						}
      	   			}
					getNewImages(Puzzle.FiveXFive, 2, possibilities);
				}
			}
			frame.validate();
			frame.repaint();

		}
	};
	
	/* This is used by TypeListener to get a more suitable set of images */
	private void getNewImages(int Type, int TypeNumber, Object[] possibilities) {
		String s = (String)JOptionPane.showInputDialog(
			frame,
			"The current images cannot be used for this size. Please select a different set",
			"Customized Dialog",
			JOptionPane.PLAIN_MESSAGE,
			null,
			possibilities,
			(String) possibilities[0]);
		
		if ((s != null) && (s.length() > 0)) {
			for (int j = 0; j < possibilities.length; j++) {
				if (s.equals(possibilities[j])) {
					for (int i = 0; i < ImageFiles.size(); i++) {
						if (s.equals(((String[]) ImageFiles.get(i))[0])) {
							updateImages(i);
						}
					}
					mainPanel.setType(Type);
					Type = TypeNumber;
				}
			}
		}
	}
}
