import java.util.Random;
import java.util.Vector;
import java.util.Date;
import javax.swing.JProgressBar;

/* This generates a puzzle. The basic process is to generate a grid and then remove numbers at random while checking that what is left
is a valid puzzle. Once enough numbers are removed then stop. If a dead end is reached then put numbers back and try again. */
public class GeneratePuzzle {
	
	static int Easy = 0;
	static int Medium = 1;
	static int Hard = 2;
	
	/* A puzzle is easy if 50% of the numbers are removed, medium if 61% are removed and hard if 71% are removed */
	static double[] Remove = {0.5, 0.61, 0.71};
	
	/* Timeout after different amounts of time for different sizes of puzzle */
	static int[] Time = {10000, 60000, 600000, 5000, 9000};
	
	static int Length = 100;
	
	int Progress;
		
	public int[] getPuzzle(int Difficulty, int Size) {
		int[] StartPosition = new int[PuzzleArrays.Sizes[Size]];
		int[] TestPosition = new int[PuzzleArrays.Sizes[Size]];
		Random R = new Random();
		Worker w = new Worker(StartPosition);
		Vector Removed = new Vector();
		Vector XRemoved = new Vector();
		Vector YRemoved = new Vector();
		Vector XYRemoved = new Vector();
		int Current;
		int Failures = 0;
		int LastPos;
		Date start;
		Date finish;
		int BestLength;
		int BestDifficultyLength;
		int XPos;
		int YPos;
		int XYPos;
		int[] BestPosition = new int[PuzzleArrays.Sizes[Size]];
		int[] BestDifficultyPosition = new int[PuzzleArrays.Sizes[Size]];
		int ToRemove = (int) (Remove[Difficulty] * PuzzleArrays.Sizes[Size]);
		int FailureIncrement = 1;
		
		/* Using symmetry constraints each number counts as two or four as it's counterparts are also removed */
		if (OptionScreen.getXAxis() && !OptionScreen.getYAxis()) {
			ToRemove = (int) (ToRemove / 1.5);
			FailureIncrement = 2;
		}
		if (OptionScreen.getYAxis() && !OptionScreen.getXAxis()) {
			ToRemove = (int) (ToRemove / 1.5);
			FailureIncrement = 2;
		}
		if (OptionScreen.getXAxis() && OptionScreen.getYAxis()) {
			ToRemove = ToRemove / 3;
			FailureIncrement = 4;
		}
		boolean BestDifficultySet = false;		
		Progress = 0;
		
		StartPosition = GenerateGrid.getFullGrid(Size);
		
		Progress = 10;
		
		for (int i = 0; i < PuzzleArrays.Sizes[Size]; i++) {
			TestPosition[i] = StartPosition[i];
		}
		start = new Date();
		finish = new Date();
		BestLength = 0;
		BestDifficultyLength = 0;
		while   ((Removed.size() < ToRemove) && 
				((finish.getTime() - start.getTime()) < Time[Size])) {
			Progress = (int) (((finish.getTime() - start.getTime()) * 90 / Time[Size]))  + 10;
			do {
				Current = R.nextInt(PuzzleArrays.Sizes[Size]);		
			} while (TestPosition[Current] == 0);
			
			TestPosition[Current] = 0;
			
			XPos = Current;
			if (OptionScreen.getXAxis()) {
				XPos = Current - (2 * (Current % PuzzleArrays.RowLength[Size])) + PuzzleArrays.RowLength[Size] - 1;
				TestPosition[XPos] = 0;
			}

			YPos = Current;
			if (OptionScreen.getYAxis()) {
				YPos = (Current % PuzzleArrays.ColumnLength[Size]) + 
						(PuzzleArrays.ColumnLength[Size] * (PuzzleArrays.ColumnLength[Size] - 
															(Current / PuzzleArrays.ColumnLength[Size]) - 1));
				TestPosition[YPos] = 0;
			}

			XYPos = Current;
			if (OptionScreen.getXAxis() && OptionScreen.getYAxis()) {
				XYPos = YPos - (2 * (YPos % PuzzleArrays.RowLength[Size])) + PuzzleArrays.RowLength[Size] - 1;
				TestPosition[XYPos] = 0;
			}
			
			
			w.setPosition(TestPosition);
		
			if (!w.solve()) {
				TestPosition[Current] = StartPosition[Current];
				if (OptionScreen.getXAxis()) {
					TestPosition[XPos] = StartPosition[XPos];
				}
				if (OptionScreen.getYAxis()) {
					TestPosition[YPos] = StartPosition[YPos];
				}
				if (OptionScreen.getXAxis() && OptionScreen.getYAxis()) {
					TestPosition[XYPos] = StartPosition[XYPos];
				}
				Failures += FailureIncrement;
			} else {
				if (BestLength < Removed.size()) {
					for (int i = 0; i < PuzzleArrays.Sizes[Size]; i++) {
						BestPosition[i] = TestPosition[i];
					}
					BestLength = Removed.size();
				}
				if (BestDifficultyLength < Removed.size()) {
					if (((Difficulty == Easy) && w.isEasy(TestPosition)) ||
						((Difficulty == Medium) && w.isMedium(TestPosition)) ||
						((Difficulty == Hard) && w.isHard(TestPosition))) {
						for (int i = 0; i < PuzzleArrays.Sizes[Size]; i++) {
							BestDifficultyPosition[i] = TestPosition[i];
						}
						BestDifficultyLength = Removed.size();
						BestDifficultySet = true;
					}
				}
				Removed.add(new Integer(Current));
				XRemoved.add(new Integer(XPos));
				YRemoved.add(new Integer(YPos));
				XYRemoved.add(new Integer(XYPos));
				Failures = 0;
			}
			if (Failures >= (PuzzleArrays.Sizes[Size] - Removed.size()) && (Removed.size() > 0)) {
				/* Go back and try again */
				LastPos = ((Integer) Removed.lastElement()).intValue();
				TestPosition[LastPos] = StartPosition[LastPos];
				if (Removed.size() > 0) {
					Removed.remove(Removed.lastElement());
					if (OptionScreen.getXAxis()) {
						LastPos = ((Integer) XRemoved.lastElement()).intValue();
						TestPosition[LastPos] = StartPosition[LastPos];
						}
					if (OptionScreen.getYAxis()) {
						LastPos = ((Integer) YRemoved.lastElement()).intValue();
						TestPosition[LastPos] = StartPosition[LastPos];
						}
					if (OptionScreen.getXAxis() && OptionScreen.getYAxis()) {
						LastPos = ((Integer) XYRemoved.lastElement()).intValue();
						TestPosition[LastPos] = StartPosition[LastPos];
					}
				}
			}
			finish = new Date();
		}
		if (w.isEasy(BestPosition)) {
			return BestPosition;
		} else {
			if (BestDifficultySet) {
				return BestDifficultyPosition;
			} else {
				return BestPosition;
			}
		}
	}
	
	public int getProgress() {
		return Progress;
	}
}