import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

/* As generating puzzles takes quite some time it is done by a separate thread so that the whole program doesn't freeze up */
class Generator extends Thread {
	Puzzle CurrentPuzzle;
	GeneratePuzzle GP;
	ControlPanel CurrentPP;
	int Difficulty;
	int Type;
	
	public Generator(ControlPanel PP, Puzzle CP, GeneratePuzzle CurrentGP, int D, int T) {
		CurrentPP = PP;
		CurrentPuzzle = CP;
		GP = CurrentGP;
		Difficulty = D;
		Type = T;
	}

	public void run() {
		try {
			CurrentPuzzle.setPuzzle(GP.getPuzzle(Difficulty, Type));
			CurrentPuzzle.setDifficulty(PuzzleArrays.Difficulty[Difficulty]);
			CurrentPP.finishGeneration();
		} catch (Exception e) {}
	}
}

/* Used to increment the progress bar while generation is going no */
class PBTimer extends Timer {
				
	JProgressBar ProgressBar;
	GeneratePuzzle GP;
	
	public PBTimer (JProgressBar PB, GeneratePuzzle CurrentGP) {
		super(1000, null);
		ProgressBar = PB;
		GP = CurrentGP;
		addActionListener(TimerListener);
	}
	
	ActionListener TimerListener = new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
			ProgressBar.setValue(GP.getProgress());
		}
	};
}

/* The control panel conatins various buttons */
class ControlPanel extends JPanel 
					implements ActionListener {
	JButton CheckButton;
	JButton ClearButton;
	JButton SolveButton;
	JButton FinishSetupButton;
	JButton GenerateButton;
	JButton GenerateGridButton;
	JButton CheckOKButton;
	Puzzle CurrentPuzzle;
	PuzzlePanel CurrentPP;
	JComboBox DifficultyLevel;
	JLabel DifficultyLabel;
	JPanel DifficultyPanel;
	String Level = "Easy";
	MainPanel Holder;
	JPanel ProgressBarPanel;
	JProgressBar ProgressBar;
	PBTimer timer;
	
	public ControlPanel(Puzzle P, PuzzlePanel PP, MainPanel MP) {
		super();
		CurrentPuzzle = P;
		CurrentPP = PP;
		Holder = MP;
		setInputMode();
	}
	
	public void finishGeneration() {
		CurrentPP.setState(CurrentPuzzle.getPuzzle());
		ProgressBar.setVisible(false);
	}
	
	public void addPuzzle(Puzzle P) {
		CurrentPuzzle = P;
	}
	
	/* These function switch between the various modes which mainly involves moving buttons around */
	public void setInputMode() {
		removeAll();
		FinishSetupButton = new JButton("Finish Setup");
		add(FinishSetupButton);
		FinishSetupButton.addActionListener(this);
		validate();
		repaint();
	}
	
	public void setSolveMode() {
		removeAll();
		setLayout(new GridLayout(3,1));
		CheckButton = new JButton("Check");
		ClearButton = new JButton("Clear");
		SolveButton = new JButton("Solve");
		add(CheckButton);
		add(ClearButton);
		add(SolveButton);
		CheckButton.addActionListener(this);
		ClearButton.addActionListener(this);
		SolveButton.addActionListener(this);
		validate();
		repaint();
	}
	
	public void setGenerateMode() {
		String[] Levels = {"Easy", "Medium", "Hard"};
		removeAll();
		setLayout(new GridLayout(6,1));
		GenerateButton = new JButton("Generate Puzzle");
		add(GenerateButton);
		GenerateButton.addActionListener(this);
		ProgressBarPanel = new JPanel();
		ProgressBar = new JProgressBar(0, 100);
		ProgressBar.setValue(0);
		ProgressBar.setStringPainted(true);
		ProgressBar.setVisible(false);
		ProgressBarPanel.add(ProgressBar);
		add(ProgressBarPanel);
		DifficultyPanel = new JPanel(new GridLayout(2,1));
		DifficultyLevel = new JComboBox(Levels);
		DifficultyLevel.addActionListener(this);
		DifficultyLabel = new JLabel("Difficulty");
		DifficultyLabel.setHorizontalAlignment(JLabel.CENTER);
		DifficultyPanel.add(DifficultyLabel);
		DifficultyPanel.add(DifficultyLevel);
		add(DifficultyPanel);
		GenerateGridButton = new JButton("Generate Grid");
		add(GenerateGridButton);
		GenerateGridButton.addActionListener(this);
		CheckOKButton = new JButton("Check OK");
		add(CheckOKButton);
		CheckOKButton.addActionListener(this);
		FinishSetupButton = new JButton("Finish Generation");
		add(FinishSetupButton);
		FinishSetupButton.addActionListener(this);
		validate();
		repaint();
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == CheckButton) {
			CurrentPP.checkCorrect(CurrentPuzzle);
		}
		
		if (e.getSource() == ClearButton) {
			CurrentPP.setState(CurrentPuzzle.getPuzzle());
		}
		
		if (e.getSource() == SolveButton) {
			CurrentPP.setState(CurrentPuzzle.getSolution());
		}
		
		if (e.getSource() == FinishSetupButton) {
			try {
				CurrentPuzzle.setPuzzle(CurrentPP.getCurrentState());
				if (CurrentPuzzle.solvePuzzle()) {
					CurrentPP.setFixed();
					Holder.setSolveMode();
				} else {
					JOptionPane.showMessageDialog(this,
						"This Sudoku puzzle cannot be solved, please try again",
						"Invalid Puzzle",
						JOptionPane.ERROR_MESSAGE);
    			}
			}
			catch (InsolublePuzzleError ipe) {
				JOptionPane.showMessageDialog(this,
					"This Sudoku puzzle cannot be solved, please try again",
					"Invalid Puzzle",
					JOptionPane.ERROR_MESSAGE);
			}
			catch (InvalidPuzzleError ipe) {
				JOptionPane.showMessageDialog(this,
					"Whoops! Something has gone wrong",
					"Program Error",
					JOptionPane.ERROR_MESSAGE);
			}
		}
		
		if (e.getSource() == GenerateButton) {
			GeneratePuzzle GP = new GeneratePuzzle();
			if (Level.equals("Easy")) {
				try {
					ProgressBar.setVisible(true);
					ProgressBar.setValue(0);
					timer = new PBTimer(ProgressBar, GP);
					timer.start();
					ProgressBar.setIndeterminate(true);
					Generator G = new Generator(this, CurrentPuzzle, GP, GeneratePuzzle.Easy, CurrentPuzzle.getGridType());
					G.start();
		
				}
				catch (Exception except) {
					except.printStackTrace();
					JOptionPane.showMessageDialog(this,
						"Whoops! Something has gone wrong",
						"Program Error",
						JOptionPane.ERROR_MESSAGE);
				}
			} else {
				if (Level.equals("Medium")) {
					try {
						ProgressBar.setVisible(true);
						ProgressBar.setValue(0);
						timer = new PBTimer(ProgressBar, GP);
						timer.start();
						Generator G = new Generator(this, CurrentPuzzle, GP, GeneratePuzzle.Medium, CurrentPuzzle.getGridType());
						G.start();
}
					catch (Exception except) {
						except.printStackTrace();
						JOptionPane.showMessageDialog(this,
							"Whoops! Something has gone wrong",
							"Program Error",
							JOptionPane.ERROR_MESSAGE);
					}
					CurrentPP.setState(CurrentPuzzle.getPuzzle());
				} else {
					try {
						ProgressBar.setVisible(true);
						ProgressBar.setValue(0);
						timer = new PBTimer(ProgressBar, GP);
						timer.start();
						Generator G = new Generator(this, CurrentPuzzle, GP, GeneratePuzzle.Hard, CurrentPuzzle.getGridType());
						G.start();
					}
					catch (Exception except) {
						except.printStackTrace();
						JOptionPane.showMessageDialog(this,
							"Whoops! Something has gone wrong",
							"Program Error",
							JOptionPane.ERROR_MESSAGE);
					}
					CurrentPP.setState(CurrentPuzzle.getPuzzle());
				}
			}
		}
		
		if (e.getSource() == DifficultyLevel) {
			Level = (String) ((JComboBox)e.getSource()).getSelectedItem();
		}
		
		if (e.getSource() == GenerateGridButton) {
			try {
				CurrentPuzzle.setPuzzle((new GenerateGrid()).getFullGrid(CurrentPuzzle.getGridType()));
			}
			catch (InvalidPuzzleError ipe) {
				JOptionPane.showMessageDialog(this,
					"Whoops! Something has gone wrong",
					"Program Error",
					JOptionPane.ERROR_MESSAGE);
			}

			CurrentPP.setState(CurrentPuzzle.getPuzzle());
		}
		
		if (e.getSource() == CheckOKButton) {
			try {
			if (CurrentPuzzle.solvePuzzle()) {
				JOptionPane.showMessageDialog(this,
					"This Sudoku puzzle can be solved",
					"Valid Puzzle",
					JOptionPane.INFORMATION_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(this,
					"This Sudoku puzzle cannot be solved",
					"Invalid Puzzle",
					JOptionPane.INFORMATION_MESSAGE);
			}
			}
			catch (InsolublePuzzleError ipe) {
				JOptionPane.showMessageDialog(this,
					"This Sudoku puzzle cannot be solved",
					"Invalid Puzzle",
					JOptionPane.INFORMATION_MESSAGE);
			}	
		}
   	}
}

/* The mainPanel contains the ControlPanel and the PuzzlePanel. It mostly passes message between those and the Display */ 
public class MainPanel extends JPanel {
	PuzzlePanel PP;
	ControlPanel CP;
	static Puzzle CurrentPuzzle;
	String Mode;
	Display MainDisplay;

	public MainPanel(Display D) {
		super();
		MainDisplay = D;
		Mode = "input";
		CurrentPuzzle = new Puzzle(Puzzle.ThreeXThree);
		setBackground(Color.green);
		setLayout(new BorderLayout());
		PP = new PuzzlePanel(CurrentPuzzle.getPuzzle(), false);
		CP = new ControlPanel(CurrentPuzzle, PP, this);
		add(PP, BorderLayout.CENTER);
		add(CP, BorderLayout.EAST);
		setState(Mode);
	}
	
	public void addPuzzle(Puzzle NewPuzzle) {
		CurrentPuzzle = NewPuzzle;
		PP.setState(CurrentPuzzle.getPuzzle());
		CP.addPuzzle(CurrentPuzzle);
	}
	
	public void addPuzzle(Puzzle NewPuzzle, boolean IncludeAttempted) {
		CurrentPuzzle = NewPuzzle;
		PP.setState(CurrentPuzzle.getPuzzle());
		if (IncludeAttempted) {
			PP.setAttempted(CurrentPuzzle.getAttempted());
		}
		CP.addPuzzle(CurrentPuzzle);
	}
	
	public static Puzzle getPuzzle() {
		return CurrentPuzzle;
	}
	
	public void setSolveMode() {
		CP.setSolveMode();
		PP.setState(CurrentPuzzle.getPuzzle());
		CP.addPuzzle(CurrentPuzzle);
		MainDisplay.setModeText("Mode: Solve");
		Mode = "solve";
		validate();
		repaint();
	}
	
	public void setState(String S) {
		if (!(S.equals(Mode))) {
			Mode = S;
			if (S.equals("input")) {
				CurrentPuzzle.setBlank();
				CP.setInputMode();
				PP.setState(CurrentPuzzle.getPuzzle());
				CP.addPuzzle(CurrentPuzzle);
				MainDisplay.setModeText("Mode: Input");
				validate();
				repaint();
			}
			if (S.equals("generate")) {
				CurrentPuzzle.setBlank();
				CP.setGenerateMode();
				PP.setState(CurrentPuzzle.getPuzzle());
				CP.addPuzzle(CurrentPuzzle);
				MainDisplay.setModeText("Mode: Generate");
				validate();
				repaint();
			}
			if (S.equals("solve")) {
				CurrentPuzzle.setBlank();
				CP.setSolveMode();
				PP.setState(CurrentPuzzle.getPuzzle());
				CP.addPuzzle(CurrentPuzzle);
				MainDisplay.setModeText("Mode: Solve");
				validate();
				repaint();
			}
		}
	}
	
	public int[] getAttempted() {
		return PP.getAttempted();
	}
	
	public void updateImages() {
		PP.setState(PP.getCurrentState());
		validate();
		repaint();
	}
	
	public void setType(int Type) {
		CurrentPuzzle.setType(Type);
		PP.setType(Type);
	}
}