/* This sets up a whole bunch of arrays that map the individual numbers, which read from left to right
 and top to bottom into the rows, columns and groups that make up a sudoku puzzle */
public class PuzzleArrays {	
	
	static int[][][] Rows;
	
	static int[][][] Columns;
	
	static int[][][] Groups;		
	
	static int[][] GroupID;		
	
	static int[][] DisplayID;	 
	
	static int[][] XCoord;
	
	static int[][] YCoord;
	
	/* The order is 3x3, 4x4, 5x5, 2x2, 4x2 */
	static int[] Sizes = {81, 256, 625, 16, 64};	 
	static int[] NumberOfDigits = {9, 16, 25, 4, 8};
	static int[] RowLength = {9, 16, 25, 4, 8};
	static int[] ColumnLength = {9, 16, 25, 4, 8};
	static int[] GroupLength = {9, 16, 25, 4, 8};
	static int[] GroupWidth = {3, 4, 5, 2, 4};
	static int[] GroupDepth = {3, 4, 5, 2, 2};
	static String[] Difficulty = {"Easy", "Medium", "Hard"};
	
	static {
		Rows = new int[Sizes.length][][];
		for (int i = 0; i < Sizes.length; i++) {
			Rows[i] = new int[RowLength[i]][];
			for (int j = 0; j < RowLength[i]; j++) { 
				Rows[i][j] = new int[RowLength[i]];
				for (int k = 0; k < RowLength[i]; k++) {
					Rows[i][j][k] = k + (j * RowLength[i]);
				}
			}
		}

		Columns = new int[Sizes.length][][];
		for (int i = 0; i < Sizes.length; i++) {
			Columns[i] = new int[ColumnLength[i]][];
			for (int j = 0; j < ColumnLength[i]; j++) { 
				Columns[i][j] = new int[ColumnLength[i]];
				for (int k = 0; k < ColumnLength[i]; k++) {
					Columns[i][j][k] = k * ColumnLength[i] + j;
				}
			}
		}

		/* Groups are the 3x3 squares and so on */
		int GR = 0;
		int GD = 0;
		Groups = new int[Sizes.length][][];
		for (int i = 0; i < Sizes.length; i++) {
			GD = 0;
			GR = 0;
			Groups[i] = new int[GroupLength[i]][];
			for (int j = 0; j < GroupLength[i]; j++) { 
				Groups[i][j] = new int[GroupLength[i]];
				for (int k = 0; k < (GroupLength[i] / GroupWidth[i]); k++) {
					for (int l = 0; l < GroupWidth[i]; l++) {
						Groups[i][j][k * GroupWidth[i] + l] = Rows[i][k + GD][l + GR];
					}
				}
				GD += GroupDepth[i];
				if (GD > (GroupLength[i] - GroupDepth[i])) {
					GD = 0;
					GR += GroupWidth[i];
				}
			}
		}		
		
		/* This is the reverse mapping to Groups */
		int GID = 0;
		int Count = 0;
		GroupID = new int[Sizes.length][];
		for (int i = 0; i < Sizes.length; i++) {
			GID = 0;
			Count = 0;
			GroupID[i] = new int[Sizes[i]];
			for (int j = 0; j < Sizes[i]; j++) {
				GroupID[i][j] = GID;
				Count++;
				if (Count == GroupWidth[i]) {
					Count = 0;
					GID += GroupWidth[i];
					if ((GID > GroupLength[i]) || 
						(GID == GroupLength[i] && (((j / GroupLength[i]) / GroupDepth[i]) == 0)))
						GID = (j / GroupLength[i] + 1) / GroupDepth[i];
				}
			}
		}
		
		Count = 0;
		int Counter = 0;
		int DID = 0;
		int DIG = 0;
		DisplayID = new int[Sizes.length][];
		for (int i = 0; i < Sizes.length; i++) {
			DID = 0;
			DIG = 0;
			Count = 0;
			Counter = 0;
			DisplayID[i] = new int[Sizes[i]];
			for (int j = 0; j < Sizes[i]; j++) {
				DisplayID[i][j] = Count;
				if (Counter % GroupWidth[i] == GroupWidth[i] - 1) {
					Count++;
				}
				if (Counter % NumberOfDigits[i] == NumberOfDigits[i] - 1) {
					Count = DID;
					DIG++;
					if (DIG % GroupDepth[i] == GroupDepth[i] - 1) {
						DID += GroupDepth[i];
					}
				}
				Counter++;
			}
		}

		/* Used to display coords for the notes */
		XCoord = new int[Sizes.length][];
		for (int i = 0; i < Sizes.length; i++) {
			XCoord[i] = new int[Sizes[i]];
			for (int j = 0; j < Sizes[i]; j++) {
				XCoord[i][j] = j % RowLength[i] + 1;
			}
		}
		
		YCoord = new int[Sizes.length][];
		for (int i = 0; i < Sizes.length; i++) {
			YCoord[i] = new int[Sizes[i]];
			for (int j = 0; j < Sizes[i]; j++) {
				YCoord[i][j] = j / RowLength[i] + 1;
			}
		}
	}				
}