import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.accessibility.Accessible;

/* This allows notes to be input for each square */
class NotesFrame extends JFrame 
					implements ActionListener{
	JPanel MainPanel;
	JPanel ButtonsPanel;
	JTextArea TextBox;
	JButton OKButton;
	JButton CancelButton;
	Digit Owner;
	
	public NotesFrame(Digit O, String Title) {
		super(Title);
		Owner = O;
		MainPanel = new JPanel(new GridLayout(2,1));
		ButtonsPanel = new JPanel(new GridLayout(1,2));
		TextBox = new JTextArea(Owner.getNotes());
		OKButton = new JButton("OK");
		OKButton.addActionListener(this);
		CancelButton = new JButton("Cancel");
		CancelButton.addActionListener(this);
		ButtonsPanel.add(OKButton);
		ButtonsPanel.add(CancelButton);
		MainPanel.add(TextBox);
		MainPanel.add(ButtonsPanel);
		getContentPane().add(MainPanel);
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		pack();	
		setVisible(true);	
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == OKButton) {
			Owner.setNotes(TextBox.getText());
			setVisible(false);
			dispose();
		}
		if (e.getSource() == CancelButton) {
			setVisible(false);
			dispose();
		}
	}
}

/* This is basically the visibale bit of a square */
class DigitPanel extends JPanel {
	static ImageIcon[] Icons;
	static Image image;

	public static void setInitialImages() {
		String[] InitialImages = Display.getFirstImages();
		Icons = new ImageIcon[InitialImages.length];
		/* InitialImages is off by one because of the images title */
		for (int i = 1; i < InitialImages.length; i++) {
			Icons[i - 1] = new ImageIcon(InitialImages[i]);
			Icons[i - 1].setImage(Icons[i - 1].getImage().getScaledInstance(30, 30, Image.SCALE_SMOOTH));
		}
	}
	
	int Value;
	ImageIcon icon;
	
	public static void changeImages(String[] ImageFiles) {
		Icons = new ImageIcon[ImageFiles.length];
		for (int i = 1; i < ImageFiles.length; i++) {
			Icons[i - 1] = new ImageIcon(ImageFiles[i]);
			Icons[i - 1].setImage(Icons[i - 1].getImage().getScaledInstance(30, 30, Image.SCALE_SMOOTH));
		}
	}
	
	private void setIcon(int I) {
		icon = Icons[I];
		Value = I;
	}
	
	/* Really don't need both of these */
	public DigitPanel(int InitialValue, boolean Printable) {
		super();
		if (Printable) setBackground(Color.white);
		setIcon(InitialValue);
		setPreferredSize(new Dimension(30,30));
		repaint();
	}
	
	public DigitPanel(int InitialValue, int Size, boolean Printable) {
		super();
		if (Printable) setBackground(Color.white);
		setIcon(InitialValue);
		setPreferredSize(new Dimension(Size,Size));
		repaint();
	}
	
	public void setCaption(int C) {
		setIcon(C);
		repaint();
	}
	
	public int getCaption() {
		return Value;
	}
	
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		//  Scale image to size of component
		Dimension d = getSize();
		g.drawImage(icon.getImage(), 0, 0, d.width, d.height, null);
	}
}

/* This does all the control stuff for the square */
class Digit extends JPanel 
			implements 	MouseListener,
						FocusListener,
						KeyListener,
						Accessible {
	
	DigitPanel DPanel;	
	Color DefaultColour;
	Color DeselectedColour;
	boolean Fixed;
	int Type;
	String Notes;
	NotesFrame NotesWindow;
	int XCoord;
	int YCoord;
	
	public Digit(int InitialValue, int T, boolean Printable) {
		super(new GridLayout(1,1));
		
		/* If it's going to be printed we don't need the control stuff */
		if (!Printable) {	
			Type = T;
			
			if (InitialValue == 0) {
				Fixed = false;
			} else {
				Fixed = true;
			}
			setFocusable(true);
			/* if it's fixed then it's one of the givens, and shouldn't be changed */
			if (!Fixed) {
				addMouseListener(this);
				addFocusListener(this);
				addKeyListener(this);
			}
			DefaultColour = getBackground();
			DeselectedColour = DefaultColour;
		}
		setBorder(BorderFactory.createLineBorder(Color.black));
		DPanel = new DigitPanel(InitialValue, Printable);
		add(DPanel);
	}
	
	public void mouseClicked(MouseEvent e) {
		requestFocusInWindow();
	}
	
	public void mouseEntered(MouseEvent e) { }
	
	public void mouseExited(MouseEvent e) { }
	
	public void mousePressed(MouseEvent e) { 
		if (e.getButton() == MouseEvent.BUTTON3) {
			/* use the coords to identify where the notes come from */
			NotesWindow = new NotesFrame(this, 
							(new Integer(XCoord).toString() + " " + new Integer(YCoord).toString()));
			NotesWindow.setLocationRelativeTo(this);
		}
	}
	
	public void mouseReleased(MouseEvent e) { }
	
	public void focusGained(FocusEvent e) { 
		setSelected();
	}
	
	public void focusLost(FocusEvent e) {
		setDeselected();
	}
	
	public void keyTyped(KeyEvent e) {
		setCaption(e.getKeyChar());
	}

	public void keyPressed(KeyEvent e) { }

	public void keyReleased(KeyEvent e) { }
	
	public void setSelected() {
		DPanel.setBackground(Color.blue);
	}	
	
	public void setDeselected() {
		DPanel.setBackground(DeselectedColour);
	}
	
	public void setDefaultColour() {
		DPanel.setBackground(DefaultColour);
	}
	
	public void setCaption(char C) {
	
		if (Keys.getValue(C) <= PuzzleArrays.NumberOfDigits[Type]) {
			DPanel.setCaption(Keys.getValue(C));
		} else {
			DPanel.setCaption(0);
		}
	}
	
	public int getValue() {
		return DPanel.getCaption();
	}
	
	public void setValue(int v) {
		DPanel.setCaption(v);
		DeselectedColour = DefaultColour;
	}
	
	public void setCorrect() {
		DPanel.setBackground(Color.green);
		DeselectedColour = Color.green;
	}
	
	public void setIncorrect() {
		if (DPanel.getCaption() != 0) {
			DPanel.setBackground(Color.red);
			DeselectedColour = Color.red;
		}
	}
	
	public void setFixed() {
		Fixed = true;
		removeMouseListener(this);
		removeFocusListener(this);
		removeKeyListener(this);
	}
	
	public void setUnfixed() {
		Fixed = false;
		addMouseListener(this);
		addFocusListener(this);
		addKeyListener(this);
	}
	
	public void setNotes(String S) {
		MainPanel.getPuzzle().setNotes(XCoord, YCoord, S);
		Notes = S;
	}
	
	public void updateNotes() {
		Notes = MainPanel.getPuzzle().getNotes(XCoord, YCoord);
	}
	
	public String getNotes() {
		return Notes;
	}
	
	public void setCoords(int X, int Y) {
		XCoord = X;
		YCoord = Y;
	}
}

/* This contains all the squares */
public class PuzzlePanel extends JPanel {
	Digit[] Digits;
	JPanel[] DigitPanels;
	int Type;
	boolean isPrintable = false;

	public PuzzlePanel(int[] InitialState, boolean Printable) {
		super();
		
		Type = 0;
		isPrintable = Printable;
		update(Printable);
	}
	
	public PuzzlePanel(int[] InitialState, int StartType, boolean Printable) {
		super();
		
		Type = StartType;
		isPrintable = Printable;
		update(Printable);
	}
	
	private void update(boolean Printable) {
		setLayout(new GridLayout( PuzzleArrays.GroupWidth[Type],
							  PuzzleArrays.GroupDepth[Type]));
		/* The DigitPanels contain the 3x3 or whatever squares */
		DigitPanels = new JPanel[PuzzleArrays.GroupLength[Type]];
				
		for (int i = 0; i < PuzzleArrays.GroupLength[Type]; i++) {
			DigitPanels[i] = new JPanel(new GridLayout( PuzzleArrays.GroupDepth[Type],
														PuzzleArrays.GroupWidth[Type]));
			DigitPanels[i].setBorder(BorderFactory.createLineBorder(Color.black));
			add(DigitPanels[i]);
		}
		Digits = new Digit[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			Digits[i] = new Digit(0, Type, Printable);
			/* This fits the square into the correct panel */
			DigitPanels[PuzzleArrays.DisplayID[Type][i]].add(Digits[i]);
			Digits[i].setCoords(PuzzleArrays.XCoord[Type][i], PuzzleArrays.YCoord[Type][i]);
			Digits[i].updateNotes();
		}
	}
	
	public int[] getCurrentState() {
		int[] State = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			State[i] = Digits[i].getValue();
		}
		return State;
	}
	
	/* Sets the correct guesses green */
	public void setCorrect(int[] Correct) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			if (Correct[i] == 1) {
				Digits[i].setCorrect();
			}
		}
	}
	
	/* Sets the incorrect guesses red */
	public void setIncorrect(int[] Incorrect) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			if (Incorrect[i] == 1) {
				Digits[i].setIncorrect();
			}
		}
	}
	
	public void setState(int[] State) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			Digits[i].setValue(State[i]);
			Digits[i].setDefaultColour();
		}
	}
	
	public void setFixed() {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			if (Digits[i].getValue() != 0) {
				Digits[i].setFixed();
			}
		}
	}
	
	public int[] getAttempted() {
		int[] Attempted = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			Attempted[i] = Digits[i].getValue();
		}
		return Attempted;
	}
	
	public void setAttempted(int[] Attempted) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			Digits[i].setValue(Attempted[i]);
		}
	}
	
	/* This checks that the current state of the puzzle is legit */
	public void checkCorrect(Puzzle CurrentPuzzle) {
		int[] Correct = new int[PuzzleArrays.Sizes[Type]];
		int[] Incorrect = new int[PuzzleArrays.Sizes[Type]];
		int[] CurrentState = getCurrentState();
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			if (CurrentPuzzle.getPuzzle()[i] != 0) {
				Correct[i] = 0;
				Incorrect[i] = 0;
			} else {
				if (CurrentState[i] == CurrentPuzzle.getSolution()[i]) {
					Correct[i] = 1;
				} else {
					Incorrect[i] = 1;
				}
			}
		}
		setCorrect(Correct);
		setIncorrect(Incorrect);
	}
	
	public void setType(int T) {
		Type = T;
		removeAll();
		update(isPrintable);	
	}
}