import java.util.Vector;
import java.util.Date;

/* This is the class that actually solves the puzzle. The method of doing this is to first check for any
empty squares that can only accept one number, then looking at pairs of numbers so that if two squares are
the only two squares in a grouping that contain a pair of numbers then any other numbers that that those
squares contain are invalid and can be removed */
public class Worker {
	int StartPosition[];
	
	int Type = 0;
	
	Worker(int[] Start) {
		for (int i = 0; i < PuzzleArrays.Sizes.length; i++) {
			if (PuzzleArrays.Sizes[i] == Start.length) {
				Type = i;
			}
		}
		StartPosition = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			StartPosition[i] = Start[i];
		}
	}
	
	public void setPosition(int[] Position) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			StartPosition[i] = Position[i];
		}
	}
	
	/* This checks that the final solution is legit */
	private boolean checkLegal(int[] CurrentPosition) {
		int Row;
		int Column;
		int Group;
		int Count;
		
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			Row = i / PuzzleArrays.RowLength[Type];
			Column = i % PuzzleArrays.RowLength[Type];
			Group = PuzzleArrays.GroupID[Type][i];
			Count = 0;
			for (int j = 0; j < 9; j++) {
				if (StartPosition[PuzzleArrays.Rows[Type][Row][j]] == StartPosition[i]) Count++;
				if (StartPosition[PuzzleArrays.Columns[Type][Column][j]] == StartPosition[i]) Count++;
				if (StartPosition[PuzzleArrays.Groups[Type][Group][j]] == StartPosition[i]) Count++;
			}
			if (Count >= 4) return false;
		}
		return true;
	}
	
	/* This is mainly for testing purposes */
	private void PrintSolution(int[] TestPosition) {
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			System.out.print("\t" + TestPosition[i]);
			if (i % PuzzleArrays.RowLength[Type] == (PuzzleArrays.RowLength[Type] - 1)) {
				System.out.print("\n");
			}
		}
	}
	
	/* This checks for the cases where only one number can fit into a square */
	private boolean AttemptSolution(boolean[][] Allowed) {
		int Row;
		int Column;
		int Group;
		boolean Repeat;
		int Counter;
		int[] Possibilities = new int[PuzzleArrays.Sizes[Type]];
		
		do {
			Repeat = false;
			for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
				if (StartPosition[i] == 0) {
					Row = i / PuzzleArrays.RowLength[Type];
					Column = i % PuzzleArrays.RowLength[Type];
					Group = PuzzleArrays.GroupID[Type][i];
					
					for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type]; j++) {
						Allowed[i][StartPosition[PuzzleArrays.Rows[Type][Row][j]]] = false;
						Allowed[i][StartPosition[PuzzleArrays.Columns[Type][Column][j]]] = false;
						Allowed[i][StartPosition[PuzzleArrays.Groups[Type][Group][j]]] = false;
					}
					
					Counter = 0;
					
					for (int j = 1; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
						if (Allowed[i][j] == true) {
							Counter++;
						}
					}
					
					if (Counter == 1) {
						for (int j = 1; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
							if (Allowed[i][j] == true) {
								StartPosition[i] = j;
							}
						}
						Repeat = true;
					}
					
				} else {
					for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
						Allowed[i][j] = false;
					}
					Allowed[i][StartPosition[i]] = true;
				}
			}
		}
		while (Repeat);

		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			if (StartPosition[i] == 0) {
				return false;
			}
		}
		return true;
	}
	
	/* The next three functions are bits of the Eliminate function that were hived off */
	private boolean checkContinue(int[] Numbers) {
		for (int i = 0; i < Numbers.length; i++) {
			if (Numbers[i] < (PuzzleArrays.RowLength[Type] - (Numbers.length - (i + 1)))) {
				return true;
			}
		}
		return false;
	}
	
	private boolean checkFound(boolean[] Found, int[] Numbers) {
		for (int i = 0; i < Numbers.length; i++) {
			if (!Found[Numbers[i]]) {
				return false;
			}
		}
		return true;
	}
	
	private boolean checkEliminate(boolean[] Eliminate, int[] Numbers) {
		for (int i = 0; i < Numbers.length; i++) {
			if (Eliminate[Numbers[i]]) {
				return true;
			}
		}
		return false;
	}
	
	/* This function grew rather than being designed, so I don't really know how it works */
	private boolean Eliminate(int i, int[][] Selection, boolean[][] Allowed, int Size) {
		int c;
		int[] Numbers = new int[Size];
		boolean Changed = false;
		int counter = 0;
	
		for (int j = 0; j < Size; j++) {
			Numbers[j] = j + 1;
		}
		Numbers[Size - 1]--;
		while (checkContinue(Numbers)) {
			counter++;
			c = 0;
			Numbers[Size - 1]++;
			
			for (int j = Size - 1; j > 0; j--) {
				if (Numbers[j] == PuzzleArrays.RowLength[Type] + j - (Size - 2)) {
					Numbers[j - 1]++;
					for (int k = j; k < Size; k++) {
						Numbers[k] = Numbers[k - 1] + 1;
					}
				}
			}
			
			for (int k = 0; k < PuzzleArrays.RowLength[Type]; k++) {
				if (checkFound(Allowed[Selection[i][k]], Numbers)) {
					c++;
				} else {
					if (checkEliminate(Allowed[Selection[i][k]], Numbers)) {
						c = 100;
					}
				}
			}
			
			if (c == Size) {
				for (int k = 0; k < PuzzleArrays.RowLength[Type]; k++) {
					if (checkFound(Allowed[Selection[i][k]], Numbers)) {
						if (getLength(Allowed[Selection[i][k]]) != Size) {
							for (int l = 0; l < PuzzleArrays.RowLength[Type] + 1; l++) {
								Allowed[Selection[i][k]][l] = false;
							}
							for (int l = 0; l < Size; l++) {
								Allowed[Selection[i][k]][Numbers[l]] = true;
							}
							Changed = true;
						}
					}
				}
			}
		}
		return Changed;
	}
	
	
	/* The sequence repeats until it isn't making any changes */
	private void Elimination(boolean[][] Allowed, int Size) {
		boolean Repeat = true;
		
		while (Repeat) {
			Repeat = false;
			for (int i = 0; i < PuzzleArrays.RowLength[Type]; i++) {
				Repeat = (	Eliminate(i, PuzzleArrays.Rows[Type], Allowed, Size) |
							Eliminate(i, PuzzleArrays.Columns[Type], Allowed, Size) |
							Eliminate(i, PuzzleArrays.Groups[Type], Allowed, Size) | Repeat);
			}
			AttemptSolution(Allowed);
		}
	}

	
	private int getLength(boolean[] b) {
		int Counter = 0;
		for (int i = 0; i < PuzzleArrays.NumberOfDigits[Type] + 1; i++) {
			if (b[i]) Counter++;
		}
		return Counter;
	}
	
	/* These functions determine how tough it is to solve a puzzle by seeing how much calculation is needed
	to solve the puzzle */
	
	public boolean isEasy(int[] Position) {
		StartPosition = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			StartPosition[i] = Position[i];
		}
		boolean[][] Allowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int Counter;
		int Length;
		boolean[][] TempAllowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int x = 0;
	
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
				Allowed[i][j] = true;
			}
		}
	
		return AttemptSolution(Allowed);
	}
	
	public boolean isMedium(int[] Position) {
		StartPosition = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			StartPosition[i] = Position[i];
		}
		boolean[][] Allowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int Counter;
		int Length;
		boolean[][] TempAllowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int x = 0;
	
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
				Allowed[i][j] = true;
			}
		}
	
		if (AttemptSolution(Allowed)) {
			return true;
		} else {	
			for (int i = 2; i > 0; i--) {
				Elimination(Allowed, i);
			}
			return AttemptSolution(Allowed);
		}
	}
	
	public boolean isHard(int[] Position) {
		StartPosition = new int[PuzzleArrays.Sizes[Type]];
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			StartPosition[i] = Position[i];
		}
		boolean[][] Allowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int Counter;
		int Length;
		boolean[][] TempAllowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int x = 0;
	
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
				Allowed[i][j] = true;
			}
		}
	
		if (AttemptSolution(Allowed)) {
			return true;
		} else {	
			for (int i = 5; i > 0; i--) {
				Elimination(Allowed, i);
			}
			return AttemptSolution(Allowed);
		}
	}

	/* To speed things up this tries the quick methods first then moves on to the more time consuming ones */
	public boolean solve() {
		boolean[][] Allowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int Counter;
		int Length;
		boolean[][] TempAllowed = new boolean[PuzzleArrays.Sizes[Type]][PuzzleArrays.NumberOfDigits[Type] + 1];
		int x = 0;
	
		for (int i = 0; i < PuzzleArrays.Sizes[Type]; i++) {
			for (int j = 0; j < PuzzleArrays.NumberOfDigits[Type] + 1; j++) {
				Allowed[i][j] = true;
			}
		}
	
		if (AttemptSolution(Allowed)) {
			return true;
		} else {
			for (int j = 1; j < 5; j++) {	
				for (int i = j; i > 0; i--) {
					Elimination(Allowed, i);
				}
				if (AttemptSolution(Allowed)) {
					return true;
				}
			}
			if (OptionScreen.getCompleteSolve()) {
				for (int j = 1; j < (PuzzleArrays.NumberOfDigits[Type] / 2 + 1); j++) {	
					for (int i = j; i > 0; i--) {
						Elimination(Allowed, i);
					}
					if (AttemptSolution(Allowed)) {
						return true;
					}
			}
				
			}
		}
		return false;
	}
	
	public int[] getSolution() {
		return StartPosition;
	}
}